import * as GT from "@minecraft/server-gametest";
import { world, system } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";

//variables
var sender;
const hours = 1000;


//!spawn starbot command
world.beforeEvents.chatSend.subscribe((event) => {
    if(!event.sender.hasTag("StarBot User")) return;
    if(event.message.trim()=="!spawn starbot") {
        sender = event.sender;
        event.sender.runCommandAsync("function spawnfp");
    }
});

//function definations
function tridentThrow(bot, source, test) {
    const item = bot.getComponent("minecraft:inventory").container.getItem(0);
    if(typeof item=="undefined" || item.typeId!="minecraft:trident") {
        test.print("§dThe bot needs to have a trident in the first slot");
        return;
    }
    new ModalFormData()
    .title("Target Coords")
    .textField("X:", "0")
    .textField("Y:", "0")
    .textField("Z:", "0")
    .show(source)
    .then(r => {
        if(r.isCanceled) return;
        const [x, y, z] = r.formValues;
        const vector3 = {"x": Number(x), "y": Number(y), "z": Number(z)}
        const relativeBL = test.relativeLocation(vector3);
        bot.lookAtBlock(relativeBL);
        bot.useItemInSlot(0);
        test.runAfterDelay(20, () => bot.stopUsingItem())
    });
}

function ChangeGM(bot, source, test) {
    const gamemodes = ["survival", "creative", "adventure", "spectator"];
    new ModalFormData()
    .title("Change GameMode")
    .dropdown("Select", gamemodes)
    .show(source)
    .then(r => {
        if(r.isCanceled) return;
        bot.setGameMode(gamemodes[r.formValues[0]]);
        test.print("§d" + bot.name + "'s gamemode has been updated to " + gamemodes[r.formValues[0]]);
    });
}

function ChangeName(bot, source, test) {
    new ModalFormData()
    .title("Change Name")
    .textField("New Name", bot.nameTag)
    .show(source)
    .then(r => {
        if(r.isCanceled || r.length == 0) return;
        bot.nameTag = "§d" + r.formValues[0];
        test.print("§d" + bot.name + "'s name has been updated to " + r.formValues[0]);
    });
}

//main gametest
GT.register("star", "spawnfp", (test) => {
    const bot = test.spawnSimulatedPlayer({"x": 0, "y": 1, "z": 0}, "§dStarBot");
    test.print("Player spawned!");
    bot.runCommandAsync('tp @s "'+sender.name+'"');
    bot.runCommandAsync('give "' + sender.name + '" star:menu');
    bot.runCommandAsync('gamerule dodaylightcycle true');
    bot.runCommandAsync('gamerule domobspawning true');
    bot.runCommandAsync('gamerule randomtickspeed 1');
    world.afterEvents.entityDie.subscribe((event) => {
        const entity = event.deadEntity;
        if(entity.typeId != "minecraft:player" || entity.nameTag != bot.nameTag) return;
        entity.respawn();
        entity.runCommandAsync('tp @s "'+sender.name+'"');
    });
    world.afterEvents.itemUse.subscribe((event) => {
        const {source, itemStack} = event;
        if(itemStack.typeId!="star:menu" || itemStack.nameTag!=bot.nameTag.slice(2) || !source.hasTag("StarBot User")) return;
        new ActionFormData()
        .title("StarBot Menu")
        .body("§dBy @ayy_star")
        .button("Throw Trident")
        .button("Teleport to you")
        .button("Change GameMode")
        .button("Change Name")
        .button("Remove Bot")
        .show(source)
        .then(r => {
            if(r.isCanceled) return;
            else if(r.selection==0) {
                tridentThrow(bot, source, test);
            }
            else if(r.selection==1) {
                bot.runCommandAsync('tp @s "'+source.name+'"');
            }
            else if(r.selection==2) {
                ChangeGM(bot, source, test);
            }
            else if(r.selection==3) {
                ChangeName(bot, source, test);
            }
            else if(r.selection==4) {
                bot.kill();
                test.succeed();
            }
        });
    });
})
.maxTicks(hours*72000)
.structureName("star:2_1");

const over = world.getDimension('overworld');

over.runCommand('gamerule dodaylightcycle true');
over.runCommand('gamerule domobspawning true');
over.runCommand('gamerule randomtickspeed 1');
